//+------------------------------------------------------------------+
//|                                                  ExpertMoney.mqh |
//|                   Copyright 2009-2017, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#include <Expert\ExpertBase.mqh>
//+------------------------------------------------------------------+
//| Class CExpertMoney.                                              |
//| Purpose: Base class money managment.                             |
//| Derives from class CExpertBase.                                  |
//+------------------------------------------------------------------+
class CExpertMoney: public CExpertBase
{
  protected:
    //--- input parameters
    double m_percent;
    double tp_big,tp_small;

  public:
  //==================From tailing ===================================================================
    virtual double   LastValue(ENUM_TIMEFRAMES mtf,bool buy_sell){ return 0;   }
    virtual double   Get_SL(ENUM_TIMEFRAMES mtf,ENUM_POSITION_TYPE buy_sell){ return 0;   }
    virtual bool     SL_Open_Direction_Check(ENUM_TIMEFRAMES mtf){ return false; }
    virtual double   Get_TP(ENUM_TIMEFRAMES period,ENUM_POSITION_TYPE buy_sell){return 0.0;}  
    virtual void     Set_TP(double tp1,double tp2){tp_big=tp1;tp_small=tp2;}
    //---
    // virtual bool      CheckTrailingStopLong(CPositionInfo *position,double &sl,double &tp)  { return(false); }
    //  virtual bool      CheckTrailingStopShort(CPositionInfo *position,double &sl,double &tp) { return(false); }
    virtual bool K_CheckTrailingStopLong(CPositionInfo *position, uint mtf,double &sl, double &tp){return (false); }
    virtual bool K_CheckTrailingStopShort(CPositionInfo *position, uint mtf,double &sl, double &tp) {return (false);}
    virtual void OnTimer(){;}
    //========================================
  
    CExpertMoney(void);
    ~CExpertMoney(void);
    //--- methods of setting adjustable parameters
    void Percent(double percent)
    {
        m_percent = percent;
    }
    //--- method of verification of settings
    virtual bool ValidationSettings();
    //---
    virtual double CheckOpen(uint money_policy,ENUM_POSITION_TYPE buy_sell, double price,double sl,ENUM_TIMEFRAMES period){return 0.01;}
    virtual double CheckOpenLong(double price, double sl, ENUM_TIMEFRAMES period = PERIOD_H1){ return 0.1;};
    virtual double CheckOpenShort(double price, double sl, ENUM_TIMEFRAMES period = PERIOD_H1){return 0.1;};
    virtual double CheckReverse(CPositionInfo *position, double sl, ENUM_TIMEFRAMES period = PERIOD_H1);
    virtual double CheckClose(CPositionInfo *position);
    virtual double LongSL(double times, ENUM_TIMEFRAMES period = PERIOD_H1)
    {
        return NULL;
    }
    virtual double ShortSL(double times, ENUM_TIMEFRAMES period = PERIOD_H1)
    {
        return NULL;
    }
};
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
void CExpertMoney::CExpertMoney(void): m_percent(10.0){}
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
void CExpertMoney::~CExpertMoney(void){}
//+------------------------------------------------------------------+
//| Validation settings protected data.                              |
//+------------------------------------------------------------------+
bool CExpertMoney::ValidationSettings()
{
  if (!CExpertBase::ValidationSettings())
    return (false);
  //--- initial data checks
  if (m_percent < 0.0 || m_percent > 100.0)
  {
    printf(__FUNCTION__ + 
      ": percentage of risk should be in the range from 0 to 100 inclusive");
    return (false);
  }
  //--- ok
  return (true);
}

//+------------------------------------------------------------------+
//| Getting lot size for open long position.                         |
/*+------------------------------------------------------------------+
double CExpertMoney::CheckOpenLong(double price, double sl, ENUM_TIMEFRAMES
  period)
{
  if (m_symbol == NULL)
    return (0.0);
  //---
  double funlot;
  if (price == 0.0)
    funlot = m_account.MaxLotCheck(m_symbol.Name(), ORDER_TYPE_BUY, m_symbol.Ask(),  m_percent);
  else
    funlot = m_account.MaxLotCheck(m_symbol.Name(), ORDER_TYPE_BUY, price,   m_percent);
  if (funlot < m_symbol.LotsMin())
    return (0.0);
  //---
  return (m_symbol.LotsMin());
}

*/

//+------------------------------------------------------------------+
//| Getting lot size for reverse.                                    |
//+------------------------------------------------------------------+
double CExpertMoney::CheckReverse(CPositionInfo *position, double sl,
  ENUM_TIMEFRAMES period)
{
  double lots = 0.0;
  //---
  if (position.PositionType() == POSITION_TYPE_BUY)
    //lots = CheckOpenShort(m_symbol.Bid(), sl, period);
    lots = CheckOpen(0,POSITION_TYPE_SELL, m_symbol.Bid(), sl, period);//uint money_policy
  if (position.PositionType() == POSITION_TYPE_SELL)
    //lots = CheckOpenLong(m_symbol.Ask(), sl, period);
    lots = CheckOpen(0,POSITION_TYPE_BUY, m_symbol.Ask(), sl, period);//uint money_policy

  //---
  if (lots != 0.0)
    lots += position.Volume();
  //---
  return (lots);
}

//+------------------------------------------------------------------+
//| Getting lot size for close.                                      |
//+------------------------------------------------------------------+
double CExpertMoney::CheckClose(CPositionInfo *position)
{
  if (m_percent == 0.0)
    return (0.0);
  //---
  if ( - position.Profit() > m_account.Balance() *m_percent / 100.0)
    return (position.Volume());
  //---
  return (0.0);
}

//+------------------------------------------------------------------+
